(function () {
  function hasClass(elem, classes) {
    if (typeof classes === 'string') {
      classes = [classes];
    }
    var elem_classes = elem.className.split(' ');

    for (var c = 0; c < classes.length; c++) {
      if (elem_classes.indexOf(classes[c]) < 0) {
        return false;
      }
    }

    return true;
  }

  function forSiblings(elem, func) {
    if (elem.parentElement && elem.parentElement.children && elem.parentElement.children.length) {
      for (var c = 0; c < elem.parentElement.children.length; c++) {
        if (elem.parentElement.children[c] !== elem) {
          func(elem.parentElement.children[c], c, elem.parentElement.children);
        }
      }
    }
  }

  function getHeirarchy(elem) {
    var heirarchy = [];

    while (elem) {
      heirarchy.push(elem);
      elem = elem.parentElement;
    }

    return heirarchy;
  }

  function activateMenu(elem) {
    var open = getHeirarchy(elem), elems = document.querySelectorAll('.dropdown-menus .menu, .dropdown-menus .menu-item, .dropdown-menus .menu-link'), count = 0;

    for (var c = 0; c < elems.length; c++) {
      if (open.indexOf(elems[c]) >= 0) {
        elems[c].classList.add('open');
      } else {
        elems[c].classList.remove('open');
      }

      count++;
    }

    elems = document.querySelectorAll('.dropdown-menus .menu-link');
    for (var c = 0; c < elems.length; c++) {
      if (open.indexOf(elems[c]) >= 0) {
        elems[c].classList.add('open');
        forSiblings(elems[c], function (sib) {
          if (hasClass(sib, 'menu')) {
            sib.setAttribute('aria-hidden', false);
          }
        });
      } else {
        elems[c].classList.remove('open');
        forSiblings(elems[c], function (sib) {
          if (hasClass(sib, 'menu')) {
            sib.setAttribute('aria-hidden', true);
          }
        });
      }

      count++;
    }

    return count;
  }

  function clickHandler (e) {
    activateMenu(e.target);

    if (hasClass(e.target, ['menu-link', 'has-submenu', 'menu-handler'])) {
      e.preventDefault();
      e.stopPropagation();
    }
  }

  function keyupHandler (e) {
    if (e.keyCode === 27 && activateMenu()) {
      e.preventDefault();
      e.stopPropagation();
    }
  }

  Drupal.behaviors.soahMenus = {
    attach: function () {
      var elems = document.querySelectorAll('.dropdown-menus .menu-link.has-submenu');

      for (var c = 0; c < elems.length; c++) {
        elems[c].classList.add('menu-handler');
        elems[c].setAttribute('aria-haspopup', true);

        forSiblings(elems[c], function (sib) {
          if (hasClass(sib, 'menu') && !sib.getAttribute('aria-hidden')) {
            sib.setAttribute('aria-hidden', true);
          }
        });
      }

    }
  };

  document.addEventListener('click', clickHandler);
  document.addEventListener('keyup', keyupHandler);
  Drupal.behaviors.soahMenus.attach();
})();
